/*(c) 1997 N Grassly and A Rambaut
 *	Department of Zoology
 *	Oxford University
 *	pseq-gen.c
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "global.h"
#include "tree.h"
#include "pevolve.h"
#include "pmodels.h"
#include "random.h"

int numTaxa, numBases;

double gammaShape;
int numCats, rateHetero;

char *aminoAcids="ARNDCQEGHILKMFPSTWYV";

static double matrix [SQNUM_AA];
dvectpmty vector;
static double *categories;

/* prototypes */

char SetBase(double *P);
void RandomSequence(char *seq);
void MutateSequence(char *seq, double len);
void EvolveNode(TNode *anc, TNode *des, double scale);


/* functions */

void CreateSequences(TTree *tree)
{
	TNode *P;
	
	P=tree->nodeList;
	while (P!=NULL) {
		P->sequence=CAllocMem(numBases+1, "sequences", "CreateSequences", 0);
		P=P->next;
	}
	if (rateHetero==GammaRates)
		categories=CAllocMem(numBases*sizeof(double), "categories", "CreateSequences", 0);
}


void SetCategories()
{
	int i;
	
	if (rateHetero==GammaRates) {
		for (i=0; i<numBases; i++) {
			categories[i]=rndgamma(gammaShape) / gammaShape;
		}
	}
}


char SetBase(double *P)
{
	char j;
	double r;
	
	r=rndu();
	for (j=0; r>(*P) && j<NUM_AA; j++) P++;
	return j;
}


void RandomSequence(char *seq)
{
	int i;
	char *P;
	
	P=seq;
	for (i=0; i<numBases; i++) {
		*P=SetBase(addFreq);
		P++;
	}
	*P='\0';
}

void MutateSequence(char *seq, double len)
{
	int i, cat;
	double *Q;
	char *P;
	
	P=seq;
	
	if(rateHetero==GammaRates) {
		Q=categories;
		
		for (i=0; i<numBases; i++) {
			SetVector(vector, *P, (*Q)*len);
			*P=SetBase(vector);
			P++;
			Q++;
		}
	}else{
		SetMatrix(matrix, len);
		
		for (i=0; i<numBases; i++) {
			*P=SetBase(matrix+( ((short) (*P)) * NUM_AA) );
			P++;
		}
	}
}


void EvolveNode(TNode *anc, TNode *des, double scale)
{
	double len;
	
	len=des->length0*scale;
		
	memcpy(des->sequence, anc->sequence, numBases);
	MutateSequence(des->sequence, len);
	
	if (des->tipNo==-1) {
		EvolveNode(des, des->branch1, scale);
		EvolveNode(des, des->branch2, scale);
	}
}


void EvolveSequences(TTree *tree, double scale)
{
	RandomSequence(tree->root->sequence);
	
	EvolveNode(tree->root, tree->root->branch1, scale);
	EvolveNode(tree->root, tree->root->branch2, scale);
	if (!tree->rooted)
		EvolveNode(tree->root, tree->root->branch0, scale);
}


void WriteSequences(FILE *fv, TTree *tree)
{
	int i, j;
	char *P;

	fprintf(fv, " %d %d\n", numTaxa, numBases);

	for (i=0; i<tree->numTips; i++) {
		if (relaxedPhylip)
			fprintf(fv, "%s ", tree->names[i]);
		else {
			j=0;
			P=tree->names[i];
			while (j<10 && *P) {
				fputc(*P, fv);
				j++;
				P++;
			}
			while (j<10) {
				fputc(' ', fv);
				j++;
			}
		}
		P=tree->tips[i]->sequence;
		for (j=0; j<numBases; j++) {
			fputc(aminoAcids[*P], fv);
			P++;
		}
		fputc('\n', fv);
	}
}


